import com.sun.java.swing.*;
import java.awt.*;
import java.awt.event.*;

public class GlassPaneDemo {
    static private MyGlassPane myGlassPane;

    public static void main(String[] args) {
        JFrame frame = new JFrame("GlassPaneDemo");

        JRadioButton changeButton = 
                new JRadioButton("Glass pane \"visible\"");
        changeButton.setSelected(false);
        changeButton.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                myGlassPane.setVisible(e.getStateChange() 
                                       == ItemEvent.SELECTED);
            }
        });

        myGlassPane = new MyGlassPane(changeButton);
        frame.setGlassPane(myGlassPane);

        Container contentPane = frame.getContentPane();
        contentPane.setLayout(new FlowLayout());
        contentPane.add(changeButton);
        contentPane.add(new JButton("Button 1"));
        contentPane.add(new JButton("Button 2"));

        frame.pack();
        frame.setVisible(true);

        frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });
    }
}

/**
 * We have to provide our own glass pane so that it can paint.
 */
class MyGlassPane extends JComponent {
    Point point = null;

    public void paint(Graphics g) {
        if (point != null) {
            g.setColor(Color.red);
            g.fillOval(point.x - 10, point.y - 10, 20, 20);
        }
    }

    public void setPoint(Point p) {
        point = p;
    }

    public MyGlassPane(AbstractButton aButton) {
        //XXX: The following don't work.  Explain why.
        //setForeground(Color.red); 
        //setBackground(Color.red); 

        RBListener listener = new RBListener(aButton, this);
        addMouseListener(listener);
        addMouseMotionListener(listener);
    }
}

/** 
 * Listen for all events that our radio button is likely to be
 * interested in.  Redispatch them to the radio button.
 * If this were a real app, we'd probably do something about
 * key events.
 */
class RBListener extends MouseMotionAdapter
                 implements MouseListener {
    Toolkit toolkit;
    Component liveOne;
    MyGlassPane glassPane;
    boolean inButtonDrag = false;

    public RBListener(Component live, MyGlassPane pane) {
        toolkit = Toolkit.getDefaultToolkit();
        liveOne = live;
        glassPane = pane;
    }

    public void mouseMoved(MouseEvent e) {
        redispatchMouseEvent(e, false);
    }

    /*
     * We must forward at least the mouse drags that started
     * with mouse presses over the radio button.  Otherwise,
     * when the user presses the radio button then drags off,
     * the radio button isn't disarmed -- it keeps its dark
     * gray background or whatever its L&F uses to indicate
     * that the button is currently being pressed.
     */
    public void mouseDragged(MouseEvent e) {
        redispatchMouseEvent(e, false);
    }

    public void mouseClicked(MouseEvent e) {
        redispatchMouseEvent(e, false);
    }

    public void mouseEntered(MouseEvent e) {
        redispatchMouseEvent(e, false);
    }

    public void mouseExited(MouseEvent e) {
        redispatchMouseEvent(e, false);
    }

    public void mousePressed(MouseEvent e) {
        redispatchMouseEvent(e, false);
    }

    public void mouseReleased(MouseEvent e) {
        redispatchMouseEvent(e, true);
        inButtonDrag = false;
    }

    private void redispatchMouseEvent(MouseEvent e,
                                      boolean repaint) {
        Point panePoint = e.getPoint();
        Point buttonPoint = SwingUtilities.convertPoint(
                                        glassPane,
                                        panePoint, 
                                        liveOne);

        boolean inButton = liveOne.contains(buttonPoint);
        int eventID = e.getID();

        if (inButton && (eventID == MouseEvent.MOUSE_PRESSED)) {
            inButtonDrag = true;
        } 

        if (inButton || inButtonDrag) {
            liveOne.dispatchEvent(new MouseEvent(liveOne,
                                                 eventID,
                                                 e.getWhen(),
                                                 e.getModifiers(),
                                                 buttonPoint.x,
                                                 buttonPoint.y,
                                                 e.getClickCount(),
                                                 e.isPopupTrigger()));
        }

        if (repaint) {
            toolkit.beep();
            glassPane.setPoint(panePoint);
            glassPane.repaint();
        }
    }
}
